public class SimpleThreadsAlt {

    // Display a message, preceded by
    // the name of the current thread
    static void threadMessage(String message) {
        String threadName =
            Thread.currentThread().getName();
        System.out.format("%s: %s%n",
            threadName,
            message);
    }

    private static class MessageLoop implements Runnable {
        public void run() {
            String importantInfo[] = {
                    "Oh, mares eat oats",
                    "And does eat oats",
                    "And little lambs eat ivy",
                    "Kid'll eat ivy too"
                };
            for (int i = 0;
            i < importantInfo.length;
            i++) {
                //In this version, the try/catch block is inside
                //the loop, so the thread *can* continue even if interrupted.
                // Pause for 4 seconds
                try {
                    Thread.sleep(4000);
                } catch  (InterruptedException e) 
                {
                    //Two alternative behaviors:

                    //threadMessage("Oh, you want me to stop do you? Very well....");
                    //break;

                    threadMessage("Nothing will stop me!");

                }
                // Print a message (outside the try/catch block)
                threadMessage(importantInfo[i]);
            }
        }
    }

    public static void main(String args[])
    throws InterruptedException {

        // Delay, in milliseconds before
        // we interrupt MessageLoop
        // thread (default one hour).
        long patience = 1000 * 60 * 60;

        // If command line argument
        // present, gives patience
        // in seconds.
        if (args.length > 0) {
            try {
                patience = Long.parseLong(args[0]) * 500;
            } catch (NumberFormatException e) {
                System.err.println("Argument must be an integer.");
                System.exit(1);
            }
        }

        threadMessage("Starting MessageLoop thread");
        long startTime = System.currentTimeMillis();
        Thread t = new Thread(new MessageLoop());
        t.start();

        threadMessage("Waiting for MessageLoop thread to finish");
        // loop until MessageLoop
        // thread exits
        while (t.isAlive()) {
            threadMessage("Still waiting...");
            // Wait maximum of 1 second
            // for MessageLoop thread
            // to finish.
            t.join(1000);
            if (((System.currentTimeMillis() - startTime) > patience)
            && t.isAlive()) {
                threadMessage("Tired of waiting!");
                t.interrupt();
                // Shouldn't be long now
                // -- wait indefinitely
                t.join();
            }
        }
        threadMessage("Finally!");
    }
}